      program t
************************************************************************
*  
*     Name: t
*
*     Purpose: simple "hello world" test program that prints out a 
*              short output line for each MPI process.
*
*     Usage:
*
*     mpirun -np <num_processes> t
*
*     Revision History:
*
*     25-Sep-02 -- SAF; add header and inline documentation and recode 
*                       in Fortran 90.
*
************************************************************************

      implicit none

      include "mpif.h"

      integer (kind = 4), parameter :: MAX_NUM_PROCS = 1024
      integer (kind = 4), external :: getpid
      integer (kind = 4) :: my_rank, ierr, num_nodes, inx, my_pid
      integer (kind = 4), dimension(MAX_NUM_PROCS) :: pids
      integer (kind = 4) :: my_host_name_length
      integer (kind = 4), dimension(MAX_NUM_PROCS):: host_name_lengths
      character (len = MPI_MAX_PROCESSOR_NAME) :: my_host_name
      character (len = MPI_MAX_PROCESSOR_NAME), 
     &          dimension(MAX_NUM_PROCS) :: host_names

*     Begin t.

*     Initialize the MPI library and gather useful MPI releated data.

      call MPI_Init (ierr)
      call MPI_Comm_rank (MPI_COMM_WORLD, my_rank, ierr)
      call MPI_Comm_size (MPI_COMM_WORLD, num_nodes, ierr)

*     Have each process determine its PID and gather them all to the
*     master.

      my_pid = getpid()

      call MPI_Gather (my_pid, 1, MPI_INTEGER, pids, 1, MPI_INTEGER,
     &                 0, MPI_COMM_WORLD, ierr)

*     Do the same for the name of the processor on which each process
*     is executing.

      call MPI_Get_processor_name (my_host_name, my_host_name_length, 
     &                             ierr)

      call MPI_Gather (my_host_name_length, 1, MPI_INTEGER,
     &                 host_name_lengths, 1, MPI_INTEGER,
     &                 0, MPI_COMM_WORLD, ierr)
      call MPI_Gather (my_host_name, MPI_MAX_PROCESSOR_NAME, 
     &                 MPI_CHARACTER,
     &                 host_names, MPI_MAX_PROCESSOR_NAME, 
     &                 MPI_CHARACTER,
     &                 0, MPI_COMM_WORLD, ierr)

*     Print out the "hello world" lines.

      if (my_rank .eq. 0) then
         write (6, 600) num_nodes
 600     format ("Number of nodes:", i6)

         do inx = 1, num_nodes
            write (6, 601) inx - 1,
     &                     host_names(inx)(1:host_name_lengths(inx)),
     &                     pids(inx)
 601        format ("t: hello world from rank = ", i4, 
     &              ", hostname = ", a, ", pid = ", i12)

         enddo
      endif

*     Close out the MPI library and terminate the program.

      call MPI_Finalize (ierr)
      call exit (0)

*     End t.

      end

